# Federated Image Classification with Flower

## Introduction

In this GitHub code, we implement federated learning with the FLOWER framework for image classification. We will walk you through using this code for your own custom federated image classification use case.

We first start with a description of all the Python files, explaining their contents, and then go through a step-by-step process to run your custom dataset.

## Getting Started

Starting with a description of all files:

- `main.py` - Main file consisting of Flower with federated learning code.
- `main_xx.py` (e.g., `main_flta` or `main_esaim`) - Custom extensions of the main code. Mainly alters the dataset path.
- `strategy.py` - Implements plain and weighted averaging of weights.
- `config.yaml` - Contains all the parameters required to run the federated learning code.
- `train_and_test.py` - Implements centralized training for each individual client. The client to be trained is specified in `config.yaml` under `client_id`.
- `test_global_weights.py` - Tests any weights (specifically global federated weights) on the global test dataset or any other private test dataset. The paths to the folder with test images and custom weights are set here. This file uses `config_test.yaml`.

### Dataset Setup

1. Create a folder, e.g., `MNIST_DATASET`, and within that folder, create subfolders for each client. Name the folders 'Client1', 'Client2', ..., 'Clientx', and ensure each of these folders contains three subfolders: `train`, `test`, and `val`. Having a `test` folder is not compulsory, but having a `val` folder is necessary for evaluating the best weights for centralized training or early stopping.

2. Within each `train`, `test`, and `val` folder, create subfolders with names corresponding to their labels. For example, if using the MNIST dataset with 10 classes (0-9), each client folder should contain subfolders named '0', '1', ..., '9', each containing the respective images.

3. Ensure the label names are set in `main.py` within the dictionary variable `class_map_str_to_idx` (line 101).

4. For research purposes, create a **global test dataset** comprising test images from all clients. This dataset is used to test the global federated weights obtained after each communication round.

5. Specify the name of your output folder in `config.yaml`. The code will create a path within the `runs` folder as specified in `config.yaml`, saving all communication round (CR) weights for further testing and an `output.txt` file containing all test metrics for each CR weight.

## Running the Code

1. Add the label names to the `class_map_str_to_idx` dictionary as mentioned above.

2. Set the path to your dataset training directory (the folder containing 'Client1', 'Client2', etc.). The code will automatically load the train and val datasets for each client from the respective folder names.

3. Set the path to the global test dataset you wish to test the global weights with after each communication round.

4. In `config.yaml`, select the architecture (EfficientNet, VGG, ResNet, InceptionNet, DenseNet), choose the federated strategy, set `use_best_weights` to True to save the best weights out of the total number of local epochs, and configure the optimizer, local epochs, batch size, image size, etc.

With this, you can run any number of clients, provided you stick to the dataset setup as mentioned above.